/*
 * hammurapi-rules-tutorial @mesopotamia.version@
 * Hammurapi rules tutorial. 
 * Copyright (C) 2006  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://http://www.hammurapi.biz
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.rules.tutorial;

import java.io.File;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;

import javax.rules.ConfigurationException;
import javax.rules.InvalidRuleSessionException;
import javax.rules.ObjectFilter;
import javax.rules.RuleExecutionSetNotFoundException;
import javax.rules.RuleRuntime;
import javax.rules.RuleServiceProvider;
import javax.rules.RuleServiceProviderManager;
import javax.rules.RuleSessionCreateException;
import javax.rules.RuleSessionTypeUnsupportedException;
import javax.rules.StatefulRuleSession;

import biz.hammurapi.rules.Conclusion;
import biz.hammurapi.rules.tutorial.conclusions.Child;
import biz.hammurapi.rules.tutorial.conclusions.Spouse;
import biz.hammurapi.rules.tutorial.facts.Person;

import biz.hammurapi.swing.Browser;
import biz.hammurapi.xml.dom.DOMUtils;

public class Tutorial {

	/**
	 * Infers relationships.
	 * @param args
	 * @return
	 * @throws ClassNotFoundException
	 * @throws ConfigurationException
	 * @throws RuleSessionTypeUnsupportedException
	 * @throws RuleSessionCreateException
	 * @throws RuleExecutionSetNotFoundException
	 * @throws RemoteException
	 * @throws InvalidRuleSessionException
	 */
	public static List infer(String ruleSetUrl, ObjectFilter filter) throws ClassNotFoundException, ConfigurationException, RuleSessionTypeUnsupportedException, RuleSessionCreateException, RuleExecutionSetNotFoundException, RemoteException, InvalidRuleSessionException {
		String ruleServiceProviderClassName = "biz.hammurapi.rules.jsr94.FileRuleServiceProvider";
		Class.forName(ruleServiceProviderClassName);
		RuleServiceProvider serviceProvider = RuleServiceProviderManager.getRuleServiceProvider(ruleServiceProviderClassName);
		RuleRuntime runtime = serviceProvider.getRuleRuntime();
		System.out.println("Loading rule set from "+ruleSetUrl);
		StatefulRuleSession session = (StatefulRuleSession) runtime.createRuleSession(ruleSetUrl, null, RuleRuntime.STATEFUL_SESSION_TYPE);
		
		Person kate = new Person("Kate", 58, false);
		Person victor = new Person("Victor", 63, true);
		session.addObject(new Spouse(kate, victor));

		Person peter = new Person("Peter", 37, true);
		session.addObject(new Child(peter, kate));
		session.addObject(new Child(peter, victor));
		
		Person alison = new Person("Alison", 36, false);
		session.addObject(new Spouse(peter, alison));

		Person lucy = new Person("Lucy", 17, false);
		session.addObject(new Child(lucy, alison));
		
		Person nancy = new Person("Nancy", 14, false);
		session.addObject(new Child(nancy, peter));
		
		Person dan = new Person("Dan", 7, true);
		session.addObject(new Child(dan, peter));
		session.addObject(new Child(dan, alison));
		
		Person audrey = new Person("Audrey", 4, false);
		session.addObject(new Child(audrey, peter));
		session.addObject(new Child(audrey, alison));		
		
		Person tanya = new Person("Tanya", 31, false);
		Person max = new Person("Max", 32, true);
		session.addObject(new Spouse(tanya, max));
		session.addObject(new Child(tanya, kate));
		session.addObject(new Child(tanya, victor));

		Person vilma = new Person("Vilma", 14, false);
		session.addObject(new Child(vilma, tanya));
		
		Person george = new Person("George", 10, true);
		session.addObject(new Child(george, tanya));
		
		Person lisa = new Person("Lisa", 5, false);
		session.addObject(new Child(lisa, tanya));
		session.addObject(new Child(lisa, max));		
		
		System.out.println("Source facts: "+session.getObjects().size());
		
		session.executeRules();
		
		List objects = new ArrayList(filter==null ? session.getObjects() : session.getObjects(filter));
		
		System.out.println("Conclusions: "+objects.size());
		
		session.release();
		return objects;
	}

	/**
	 * Main method for the tutorial.
	 * @param args The first argument is rule set URI.
	 */
	public static void main(String[] args) throws Exception {
		System.out.println("Hammurapi rules tutorial");
		
		if (args.length!=1) {
			System.out.println("Usage: java <options> biz.hammurapi.rules.tutorial.Tutorial <rule set url>");
			System.exit(1);
		}
		
		// Exclude already known.
		ObjectFilter of = new ObjectFilter() {

			public Object filter(Object arg) {
				return arg instanceof Conclusion && ((Conclusion) arg).getDepth()==0 ? null : arg;
			}

			public void reset() {
				// TODO Auto-generated method stub
				
			}
			
		};		
		
		List objects = infer(args[0], of);				
		
		Collections.sort(
				objects,
				new Comparator() {

					public int compare(Object o1, Object o2) {
						return o1.toString().compareTo(o2.toString());
					}
					
				});
		
		Iterator it=objects.iterator();
		for (int i=1; it.hasNext(); i++) {
			System.out.println(i + ": "+it.next());
		}
		
		DOMUtils.serialize(objects, "conclusions", new File("conclusions.xml"));

		Browser.showAndExitOnClose(objects, "Family relationships");		
	}

}
