/*
 * hammurapi-rules-tutorial @mesopotamia.version@
 * Hammurapi rules tutorial. 
 * Copyright (C) 2006  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://http://www.hammurapi.biz
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.rules.tutorial.conclusions;

import biz.hammurapi.rules.Negator;
import biz.hammurapi.rules.Supercedable;

/**
 * Negates bad facts. Uses equals() to compare facts.
 * @author Pavel Vlasov
 * @revision $Revision$
 */
public class SuspectParentRelationship implements Negator, Supercedable {
	String message;
	private Parent parent;

	public SuspectParentRelationship(Parent parent, String message) {
		this.message=message;
		this.parent=parent;
	}

	public boolean negates(Object obj) {
		// Negate parent itself
		if (obj instanceof Parent) {
			return parent.getSubject().equals(((Parent) obj).getSubject()) && parent.getObject().equals(((Parent) obj).getObject());
		}
		
		
		
		// Negate child
		if (obj instanceof Child) {
			return ((Child) obj).getSubject().equals(parent.getObject()) && ((Child) obj).getObject().equals(parent.getSubject());
		}
		
		return false;
	}
	
	public String toString() {
		return "[Suspect parent relationship] "+message+": "+parent;
	}

	public boolean supercedes(Object obj) {
		return obj instanceof SuspectParentRelationship && parent.supercedes(((SuspectParentRelationship) obj).parent);
	}
	
}
