/*
 * hammurapi-rules-tutorial @mesopotamia.version@
 * Hammurapi rules tutorial. 
 * Copyright (C) 2006  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://http://www.hammurapi.biz
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.rules.tutorial.rules;

import biz.hammurapi.rules.Rule;
import biz.hammurapi.rules.tutorial.conclusions.Child;
import biz.hammurapi.rules.tutorial.conclusions.Daughter;
import biz.hammurapi.rules.tutorial.conclusions.Father;
import biz.hammurapi.rules.tutorial.conclusions.GrandDaughter;
import biz.hammurapi.rules.tutorial.conclusions.GrandFather;
import biz.hammurapi.rules.tutorial.conclusions.GrandMother;
import biz.hammurapi.rules.tutorial.conclusions.GrandSon;
import biz.hammurapi.rules.tutorial.conclusions.Mother;
import biz.hammurapi.rules.tutorial.conclusions.Parent;
import biz.hammurapi.rules.tutorial.conclusions.Son;

public class GrandRules extends Rule {
	
	/**
	 * Son of child is a grandson.
	 * @param son
	 * @param parent
	 * @return
	 */
	public GrandSon infer(Son son, Parent parent) {
		return son.getObject().equals(parent.getObject()) ? new GrandSon(son.getSubject(), parent.getSubject()) : null;
	}
	
	/**
	 * Daughter of child is a granddaughter.
	 * @param daughter
	 * @param parent
	 * @return
	 */
	public GrandDaughter infer(Daughter daughter, Parent parent) {
		return daughter.getObject().equals(parent.getObject()) 
				? new GrandDaughter(daughter.getSubject(), parent.getSubject()) 
				: null;
	}
	
	/**
	 * Father of parent is grandparent.
	 * @param child
	 * @param father
	 * @return
	 */
	public GrandFather infer(Child child, Father father) {
		return child.getObject().equals(father.getObject()) ? new GrandFather(father.getSubject(), child.getSubject()) : null;
	}
	
	/**
	 * Mother of parent is grandmother.
	 * @param child
	 * @param mother
	 * @return
	 */
	public GrandMother infer(Child child, Mother mother) {
		GrandMother grandMother = child.getObject().equals(mother.getObject()) ? new GrandMother(mother.getSubject(), child.getSubject()) : null;
		return grandMother;
	}
	
}
