/*
 * hammurapi-rules-tutorial @mesopotamia.version@
 * Hammurapi rules tutorial. 
 * Copyright (C) 2006  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://http://www.hammurapi.biz
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.rules.tutorial.rules;

import biz.hammurapi.rules.Rule;
import biz.hammurapi.rules.tutorial.conclusions.Aunt;
import biz.hammurapi.rules.tutorial.conclusions.Brother;
import biz.hammurapi.rules.tutorial.conclusions.Child;
import biz.hammurapi.rules.tutorial.conclusions.Cousin;
import biz.hammurapi.rules.tutorial.conclusions.Daughter;
import biz.hammurapi.rules.tutorial.conclusions.Husband;
import biz.hammurapi.rules.tutorial.conclusions.Nephew;
import biz.hammurapi.rules.tutorial.conclusions.Niece;
import biz.hammurapi.rules.tutorial.conclusions.Parent;
import biz.hammurapi.rules.tutorial.conclusions.Sibling;
import biz.hammurapi.rules.tutorial.conclusions.Sister;
import biz.hammurapi.rules.tutorial.conclusions.Son;
import biz.hammurapi.rules.tutorial.conclusions.Uncle;
import biz.hammurapi.rules.tutorial.conclusions.Wife;

/**
 * Infers aunt, uncle, cousin, niece, nephew
 * @author Pavel Vlasov
 * @revision $Revision$
 */
public class SecondaryRules extends Rule {
	
	{
		setMethodFactTypes(new Class[] {Child.class, Child.class, Sibling.class} , Cousin.class);
	}

	/**
	 * Infers cousin as a child or parent's sibling.
	 */
	public void infer(Child child1, Child child2, Sibling sibling) {
		if (child1.getObject().equals(sibling.getSubject()) && child2.getObject().equals(sibling.getObject())) {
			post(new Cousin(child1.getSubject(), child2.getSubject()));
		}
	}
	
	/**
	 * If A is a cousin of B then B is a cousin of A
	 * @param cousin
	 * @return
	 */
	public Cousin infer(Cousin cousin) {
		return new Cousin(cousin.getObject(), cousin.getSubject());
	}
	
	{
		setMethodFactTypes(new Class[] {Aunt.class, Parent.class} , Cousin.class);
	}

	/**
	 * Another way to infer cousin to demonstrate mutiple derivations
	 * @param aunt
	 * @param child
	 */
	public void infer(Aunt aunt, Parent parent) {
		if (aunt.getSubject().equals(parent.getSubject())) {
			post(new Cousin(aunt.getObject(), parent.getObject()));
		}
	}
	
	{
		setMethodFactTypes(new Class[] {Uncle.class, Parent.class} , Cousin.class);
	}

	/**
	 * Another way to infer cousin to demonstrate mutiple derivations
	 * @param uncle
	 * @param child
	 */
	public void infer(Uncle uncle, Parent parent) {
		if (uncle.getSubject().equals(parent.getSubject())) {
			post(new Cousin(uncle.getObject(), parent.getObject()));
		}
	}
	
	/**
	 * Sister of parent is aunt.
	 * @param sister
	 * @param parent
	 */
	public Aunt infer(Sister sister, Parent parent) {
		return sister.getObject().equals(parent.getSubject()) ? new Aunt(sister.getSubject(), parent.getObject()) : null;
	}
	
	/**
	 * Wife of uncle is aunt.
	 * @param sister
	 * @param parent
	 */
	public Aunt infer(Wife wife, Uncle uncle) {
		return wife.getObject().equals(uncle.getSubject()) ? new Aunt(wife.getSubject(), uncle.getObject()) : null;
	}
	
	/**
	 * Brother of parent is uncle.
	 * @param sister
	 * @param parent
	 */
	public Uncle infer(Brother brother, Parent parent) {
		return brother.getObject().equals(parent.getSubject()) ? new Uncle(brother.getSubject(), parent.getObject()) : null;
	}
	
	/**
	 * Husband of aunt is uncle.
	 * @param sister
	 * @param parent
	 */
	public Uncle infer(Husband husband, Aunt aunt) {
		return husband.getObject().equals(aunt.getSubject()) ? new Uncle(husband.getSubject(), aunt.getObject()) : null;
	}
	
	{
		setMethodFactTypes(new Class[] {Daughter.class, Sibling.class} , Niece.class);
	}

	/**
	 * Daughter of sibling is niece.
	 */
	public void infer(Daughter daughter, Sibling sibling) {
		if (daughter.getObject().equals(sibling.getSubject())) {
			post(new Niece(daughter.getSubject(), sibling.getObject()));
		}
	}
	
	// TODO Implement: Spouse's niece is my niece.
	
	{
		setMethodFactTypes(new Class[] {Son.class, Sibling.class} , Nephew.class);
	}

	/**
	 * Son of sibling is nephew.
	 */
	public void infer(Son son, Sibling sibling) {
		if (son.getObject().equals(sibling.getSubject())) {
			post(new Nephew(son.getSubject(), sibling.getObject()));
		}
	}
	
	// TODO Implement: Spouse's nephew is my nephew.
		
}
