/*
 * hammurapi-rules-tutorial @mesopotamia.version@
 * Hammurapi rules tutorial. 
 * Copyright (C) 2006  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://http://www.hammurapi.biz
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.rules.tutorial.rules;

import biz.hammurapi.rules.Rule;
import biz.hammurapi.rules.tutorial.conclusions.Brother;
import biz.hammurapi.rules.tutorial.conclusions.Child;
import biz.hammurapi.rules.tutorial.conclusions.Sibling;
import biz.hammurapi.rules.tutorial.conclusions.Sister;

public class SiblingRules extends Rule {	
	
	public SiblingRules() {
		setMethodFactTypes(Sibling.class , new Class[] {Brother.class, Sister.class});
		setMethodFactTypes(new Class[] {Child.class, Child.class}, Sibling.class);
	}	
		
	/**
	 * Male sibling is brother, female sibling is sister.
	 * If A is a sibling of B then B is a sibling of A.
	 * @param sibling
	 * @return Brother or Sister.
	 */
	public Sibling infer(Sibling sibling) {	
		post(new Sibling(sibling.getObject(), sibling.getSubject()));
		
		if (sibling.getSubject().isMale()) {
			if (!(sibling instanceof Brother)) {
				return new  Brother(sibling.getSubject(), sibling.getObject());
			}			
		} else {
			if (!(sibling instanceof Sister)) {
				return new Sister(sibling.getSubject(), sibling.getObject());
			}			
		}
		return null;
	}

	/**
	 * If two children have common parent then they are siblings.
	 * @param child
	 * @param anotherChild
	 */
	public void infer(Child child, Child anotherChild) {
		if (!child.getSubject().equals(anotherChild.getSubject()) 
				&& child.getObject().equals(anotherChild.getObject())) {
			post(new Sibling(child.getSubject(), anotherChild.getSubject()));
			post(new Sibling(anotherChild.getSubject(), child.getSubject()));
		}
	}
}
