/*
 * hammurapi-rules-tutorial @mesopotamia.version@
 * Hammurapi rules tutorial. 
 * Copyright (C) 2006  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://http://www.hammurapi.biz
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.rules.tutorial.rules;

import biz.hammurapi.rules.Rule;
import biz.hammurapi.rules.tutorial.conclusions.BadFact;
import biz.hammurapi.rules.tutorial.conclusions.CloseRelative;
import biz.hammurapi.rules.tutorial.conclusions.Parent;
import biz.hammurapi.rules.tutorial.conclusions.Relative;
import biz.hammurapi.rules.tutorial.conclusions.SuspectParentRelationship;

public class Validator extends Rule {
	

	public Validator() {
		super("validate", "remove", "accept");
		setMethodFactTypes(Relative.class , BadFact.class);
		setMethodFactTypes(Parent.class , SuspectParentRelationship.class);
	}
	
	/**
	 * A person cannot be a relative of self.
	 * @param relative
	 */
	public void validate(Relative relative) {
		if (relative.getSubject().equals(relative.getObject())) {
			post(new BadFact(relative, "A person cannot be a relative of self"));
		}
	}
	
	/**
	 * Close relatives can have only relationship of one type with each other.
	 * Son is not supposed to marry his mother and sister shall not marry her brother.
	 * Conclusion which was inferred later that the other is negated. Relative's compareTo() is
	 * used for comparison.
	 * @param r1
	 * @param r2
	 */
	public void validate(CloseRelative r1, CloseRelative r2) {
		if (!r1.equals(r2)
				&& ! (r1.supercedes(r2) || r2.supercedes(r1))
				&& r1.getSubject().equals(r2.getSubject())
				&& r1.getObject().equals(r2.getObject())
			) {
			CloseRelative toBeRemoved = r1.compareTo(r2) < 0 ? r2 : r1;
			post(new BadFact(toBeRemoved, "Close relatives cannot have more than one type of relationship. Relationships '"+r1+"', '"+r2+"'. Relationship '"+toBeRemoved+"' will be removed"));
		}
	}
	
	
	private int legalAge;
	
	/**
	 * Minimal age when people are allowed to marry.
	 * @param legalAge Legal age in years
	 */
	public void setLegalAge(int legalAge) {
		this.legalAge = legalAge;
	}
	
	/**
	 * Validates that difference between parent and child age is not less than legal age.
	 * @param mother
	 */
	public void validate(Parent parent) {
		if (parent.getSubject().getAge()-parent.getObject().getAge()<legalAge) {
			post(new SuspectParentRelationship(parent, parent.getSubject() + " was under legal age when "+parent.getObject()+" was born. Please verify input data."));
		}
	}


}
